<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class ETCPF_Woo_order {

	public function create_order_woo_order_page( $params ) {
		global $wpdb;
		$order_data       = [];
		$total_tax_amount = 0;

		if ( $params['was_paid'] == 0 ) {
			$order_data['status'] = 'Pending payment';
		}

		if ( $params['was_paid'] == 1 && $params['state'] == 'pending' ) {
			$order_data['status'] = 'wc-processing';
		}

		if ( $params['was_paid'] == 1 && $params['state'] == 'shipped' ) {
			$order_data['status'] = 'wc-completed';
		}

		$product_data = json_decode( $params['product_data'] );
		$product_id   = wc_get_product_id_by_sku( $product_data->sku );
		$product      = wc_get_product( $product_id );

		$receipt_id                = $params['receipt_id'];
		$reciept_match_with_orders = wc_get_orders( [ 'meta_key' => 'receipt_id', 'meta_value' => "$receipt_id" ] );

		$transaction_id       = $params['transaction_id'];
		$table                = $wpdb->prefix . 'etcpf_orders';
		$state                = $wpdb->get_var( " SELECT state from $table where transaction_id = $transaction_id" );
		$transaction_id_state = $state == 'shipped' ? true : false;

		//buyers info
		$buyers_info = $wpdb->get_results( "SELECT buyers_info from $table where transaction_id = $transaction_id " );
		$json        = json_decode( $buyers_info[0]->buyers_info );
		//shipment details
		$shipment_details = $json->shipment_details[0];

		$item = new WC_Order_Item_Shipping();
		$item->set_method_id( "flat_rate:14" ); // set an existing Shipping method rate ID
		$item->set_total( $params['shipping_cost'] ); // (optional

		if ( $reciept_match_with_orders && ! $transaction_id_state ) {

			$order = new WC_Order();
			foreach ( $reciept_match_with_orders as $receipt_order ) {
				$order->set_id( $receipt_order->id );
				$order->add_product( $product, $params['quantity'], [
					'total' => $params['price'] * $params['quantity']
				] );
			}
			$order->update_status( $order_data['status'] );
			$order->save();
			$order->calculate_totals();

		} elseif ( $reciept_match_with_orders && $transaction_id_state ) {
			$matched_status = "";

			$order          = new WC_Order();
			$matched_status = $state;

			foreach ( $reciept_match_with_orders as $receipt_order ) {
				$order->set_id( $receipt_order->id );
			}
			if ( ! $order->meta_exists( 'etcpf_carrier_name' ) && ! $order->meta_exists( 'etcpf_tracking_number' ) ) {
				$order->add_meta_data( 'etcpf_carrier_name', $shipment_details->carrier_name );
				$order->add_meta_data( 'etcpf_tracking_number', $shipment_details->tracking_code );
			}
			$order->update_status( $order_data['status'] );
			$order->save();

		} else {
			$order            = wc_create_order( $order_data );
			$total_tax_amount = $params['total_tax_cost'];

			// billing and shipping information
			$address               = [];
			$buyers_info           = json_decode( $params['buyers_info'] );
			$buyers_address        = explode( PHP_EOL, $buyers_info->formatted_address );
			$buyers_name           = explode( " ", $buyers_info->name );
			$address['country']    = end( $buyers_address );
			$address['first_name'] = $buyers_name[0];
			$address['last_name']  = end( $buyers_name );
			$address['address_1']  = $buyers_info->first_line;
			$address['address_2']  = $buyers_info->second_line;
			$address['city']       = $buyers_info->city;
			$address['state']      = $buyers_info->state;
			$address['email']      = $buyers_info->buyer_email;

			$order->set_address( $address );
			$order->set_address( $address, 'shipping' );
			$order->set_shipping_total( $params['shipping_cost'] );
//			$order->set_total( $params['price'] );
			$order->add_product( $product, $params['quantity'], [
				'total' => $params['price'] * $params['quantity']
			] );
			$order->update_status( $order_data['status'] );
			if ( $params['was_paid'] == 1 ) {
				$order->set_date_paid( date("Y-m-d H:i:s") );
			}
			$order->add_item( $item );
			$order->set_currency( $params['currency_code'] );
			$order->add_meta_data( 'receipt_id', $params['receipt_id'] ); //etsy's receipt id

			//add buyer message if applicable
			$buyer_message = $json->buyer_note;
			if ( ! empty( $buyer_message ) ) {
				$note = sprintf( "<strong>%s</strong>\n%s", "Etsy Buyer's Note", $buyer_message );
				$order->add_order_note( $note, 0 );
			}

			//add personalized message if applicable
			$variations        = $wpdb->get_results( "SELECT variations from $table where transaction_id = $transaction_id " );
			$variation_details = json_decode( $variations[0]->variations );

			foreach ( $variation_details as $key ) {
				if ( $key->formatted_name === 'Personalization' ) {
					$text    = $key->formatted_value;
					$message = sprintf( "<strong>%s</strong>\n%s", 'Etsy Personalisation Message', $text );
					$order->add_order_note( $message, 0 );
				}
			}
			$order->save();

			//add discount if applicable
			$discount_details = $json->discount_details;
			if ( $discount_details->amount === 0 ) {
				$this->addTax( $order, $params['total_tax_cost'] );
				$order->calculate_totals();
				$order->set_total( $order->get_total() + $total_tax_amount );
				$order->save();

				return;
			}
			$discount_amount = $discount_details->amount / $discount_details->divisor;
			$this->wc_order_add_discount( $order->get_id(), __( 'Discount' ), $discount_amount, $total_tax_amount );

		}
	}

	function addTax( $order, $amount ): void {
		$tax = new WC_Order_Item_Tax();
		$tax->set_rate_code( 'Etsy Tax' );
		$tax->set_tax_total( $amount );
		$order->add_item( $tax );
		$tax->save();
	}


	function wc_order_add_discount( $order_id, $title, $amount, $tax_amount ): void {
		$order    = wc_get_order( $order_id );
		$subtotal = $order->get_subtotal();
		$item     = new WC_Order_Item_Fee();

		if ( strpos( $amount, '%' ) !== false ) {
			$percentage = (float) str_replace( array( '%', ' ' ), array( '', '' ), $amount );
			$percentage = $percentage > 100 ? - 100 : - $percentage;
			$discount   = $percentage * $subtotal / 100;
		} else {
			$discount = (float) str_replace( ' ', '', $amount );
			$discount = $discount > $subtotal ? - $subtotal : - $discount;
		}

		$item->set_name( $title );
		$item->set_amount( $discount );
		$item->set_total( $discount );
		$item->save();
		$order->add_item( $item );
		$this->addTax( $order, $tax_amount );
		$order->calculate_totals();
		$order->set_total( $order->get_total() + $tax_amount );
		$order->save();
	}
}
